"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MyStromSwitchAccessory = void 0;
const dingzDaBaseAccessory_1 = require("./lib/dingzDaBaseAccessory");
/**
 * Switch Platform Accessory
 * An instance of this class is created for each accessory your platform registers
 * Each accessory may expose multiple services of different service types.
 * TODO: Implement Switch Zero in this class
 */
class MyStromSwitchAccessory extends dingzDaBaseAccessory_1.DingzDaBaseAccessory {
    constructor(_platform, _accessory) {
        var _a, _b, _c, _d, _e;
        super(_platform, _accessory);
        this._platform = _platform;
        this._accessory = _accessory;
        this.temperatureService = undefined;
        this.outletState = {
            relay: false,
            temperature: 0,
            power: 0,
        };
        // Set Base URL
        this.mystromDeviceInfo = this.device.hwInfo;
        this.log.debug('Setting informationService Characteristics ->', this.device.model);
        this.accessory
            .getService(this.platform.Service.AccessoryInformation)
            .setCharacteristic(this.platform.Characteristic.Manufacturer, 'MyStrom AG')
            .setCharacteristic(this.platform.Characteristic.AppMatchingIdentifier, 'ch.mystrom.iOSApp')
            .setCharacteristic(this.platform.Characteristic.Model, this.device.model)
            .setCharacteristic(this.platform.Characteristic.FirmwareRevision, (_a = this.mystromDeviceInfo.version) !== null && _a !== void 0 ? _a : 'N/A')
            .setCharacteristic(this.platform.Characteristic.HardwareRevision, this.mystromDeviceInfo ? 'EU/CH v2' : 'CH v1')
            .setCharacteristic(this.platform.Characteristic.SerialNumber, this.device.mac);
        this.outletService =
            (_b = this.accessory.getService(this.platform.Service.Outlet)) !== null && _b !== void 0 ? _b : this.accessory.addService(this.platform.Service.Outlet);
        this.outletService.setCharacteristic(this.platform.Characteristic.Name, (_c = this.device.name) !== null && _c !== void 0 ? _c : `${this.accessory.context.device.model} Outlet`);
        // register handlers for the On/Off Characteristic
        this.outletService
            .getCharacteristic(this.platform.Characteristic.On)
            .on("set" /* CharacteristicEventTypes.SET */, this.setOn.bind(this)) // SET - bind to the `setOn` method below
            .on("get" /* CharacteristicEventTypes.GET */, this.getOn.bind(this)); // GET - bind to the `getOn` method below
        this.outletService
            .getCharacteristic(this.platform.Characteristic.OutletInUse)
            //      .on(CharacteristicEventTypes.SET, this.setOutletInUse.bind(this)) // SET - bind to the `setOn` method below
            .on("get" /* CharacteristicEventTypes.GET */, this.getOutletInUse.bind(this)); // GET - bind to the `getOn` method below
        if (((_d = this.device.hwInfo) === null || _d === void 0 ? void 0 : _d.type) !== undefined) {
            // Switch has a temperature sensor, make it available here
            // create a new Temperature Sensor service
            this.temperatureService =
                (_e = this.accessory.getService(this.platform.Service.TemperatureSensor)) !== null && _e !== void 0 ? _e : this.accessory.addService(this.platform.Service.TemperatureSensor);
            this.temperatureService.setCharacteristic(this.platform.Characteristic.Name, 'Temperature');
            // create handlers for required characteristics
            this.temperatureService
                .getCharacteristic(this.platform.Characteristic.CurrentTemperature)
                .on("get" /* CharacteristicEventTypes.GET */, this.getTemperature.bind(this));
        }
    }
    identify() {
        this.log.debug('Identify! -> Who am I? I am', this.accessory.displayName);
    }
    // Get updated device info and update the corresponding values
    getDeviceStateUpdate() {
        return this.getDeviceReport()
            .then((report) => {
            // push the new value to HomeKit
            this.outletState = report;
            this.outletService
                .getCharacteristic(this.platform.Characteristic.On)
                .updateValue(this.outletState.relay);
            this.outletService
                .getCharacteristic(this.platform.Characteristic.OutletInUse)
                .updateValue(this.outletState.power > 0);
            if (this.temperatureService) {
                this.temperatureService
                    .getCharacteristic(this.platform.Characteristic.CurrentTemperature)
                    .updateValue(this.outletState.temperature);
            }
            return Promise.resolve();
        })
            .catch((e) => {
            return Promise.reject(e);
        });
    }
    setOn(value, callback) {
        this.log.debug('Set Characteristic On ->', value);
        this.outletState.relay = value;
        this.setDeviceState(callback);
    }
    getOn(callback) {
        var _a;
        const isOn = (_a = this.outletState) === null || _a === void 0 ? void 0 : _a.relay;
        this.log.debug('Get Characteristic On ->', isOn);
        callback(this.reachabilityState, isOn);
    }
    getTemperature(callback) {
        var _a;
        const temperature = (_a = this.outletState) === null || _a === void 0 ? void 0 : _a.temperature;
        this.log.debug('Get Characteristic Temperature ->', temperature, '° C');
        callback(this.reachabilityState, temperature);
    }
    getOutletInUse(callback) {
        var _a;
        const inUse = ((_a = this.outletState) === null || _a === void 0 ? void 0 : _a.power) > 0;
        this.log.debug('Get Characteristic OutletInUse ->', inUse);
        callback(this.reachabilityState, inUse);
    }
    setDeviceState(callback) {
        const relayUrl = `${this.baseUrl}/relay?state=${this.outletState.relay ? '1' : '0'}`;
        this.request
            .get(relayUrl)
            .catch(this.handleRequestErrors.bind(this))
            .finally(() => {
            callback(this.reachabilityState);
        });
    }
    async getDeviceReport() {
        const reportUrl = `${this.baseUrl}/report`;
        return await this.request.get(reportUrl).then((response) => {
            return response.data;
        });
    }
}
exports.MyStromSwitchAccessory = MyStromSwitchAccessory;
//# sourceMappingURL=myStromSwitchAccessory.js.map